//======================================================================
// variAnt_follow_objects.ino 
//
// This example uses some basic functions of the VariAnt library.
// It shows how to make the Robot-Ant follow a dark object.
// The constant values DIFF_REDUCTION and SUM_THRESHOLD may have 
// to be adjusted individually, depending on the setup of sensors.
// For more information, go to: https://www.variobot.com/variant
//
// created  May     2022 by Tino Werner
// modified October 2022 by Tino Werner
// Copyright (c)    2022 VARIOBOT
//======================================================================

#include <VariAnt.h>

//#define TEST

const int DIFF_REDUCTION = 2;   // reduction factor for the turning speed
const int SUM_THRESHOLD  = 750; // threshold between forward and reverse running

int diff;                       // difference of sensor values of the feelers
int sum;                        // sum of sensor values of the feelers and the jaw claws

VariAnt ant; 

void setup()
{ 
  #ifdef TEST
    Serial.begin(57600);
  #endif

  ant.setup(2); // set a higher acceleration value for faster reaction (default is 1)
}

void loop()
{ 
  ant.update(); 

  diff = (ant.getSensor(R0)-ant.getSensor(L0)) / DIFF_REDUCTION; 
  sum  =  ant.getSensor(R2)+ant.getSensor(L2)+
          ant.getSensor(R0)+ant.getSensor(L0) - SUM_THRESHOLD; 
  
  if(abs(diff) > abs(sum)) // decide whether to turn or run
  {
    ant.turn(diff > 0); // turning direction depends on the sign of the difference
    ant.setSpeed(min(abs(diff), 255)); 
  } else {
    ant.run(sum > 0); // running direction depends on the sign of the sum value
    ant.setSpeed(min(abs(sum), 255));
  }

  #ifdef TEST
    ant.printSensors(500); // Print the 9 sensor values (parameter=interval)
  #else
    ant.showSensors(); // 2 PWM LEDs show the sensor values of L0 and R0
  #endif
}
