//==================================================
// AntMotor.h 
// Arduino library to control the variAnt Robot-Ant
// Copyright (c) 2022 Tino Werner / VARIOBOT
//==================================================

#ifndef AntMotor_h
#define AntMotor_h

#include <Arduino.h>

class AntMotor 
{  
public: 
  
  static const bool FORWARD  = 0;                   // forward direction 
  static const bool BACKWARD = 1;                   // backward direction 

  AntMotor (byte pinA, byte pinB, byte pinR);       // Constructor
  void setup(byte acceleration, byte syncSlowDown); // setup for acceleration and syncSlowDown values

  void syncSteps(AntMotor* otherMotor);             // synchronizing with the other motor
  void updateSpeed();                               // controls the desired speed change 
  
  byte getSpeed();                                  // gets the absolute target speed of the motor
  void setSpeed(byte newSpeed);                     // sets the desired speed of the motor
  void setDirection(bool dir);                      // sets the desired direction of the motor
  
  void stepBack(byte steps);                        // sets a number of steps to be taken backwards
  void slowDown();                                  // slows down the speed until the next step has done

private:
  
  enum pos {GROUND,REAR,TOP,FRONT}; // identifiers for the (middle) leg positions

  byte pinA;                        // Forward pin for the motor 
  byte pinB;                        // Backward pin for the motor 
  byte pinR;                        // pin for the reedSwitch

  byte acceleration = 1;            // speed change per millisecond - best between 1 and 10 
  byte syncSlowDown = 2;            // speed reduction factor for synchronizing the motors (>1)
    
  pos  position;                    // position of the (middle) leg
  bool reed;                        // reedSwitch input: on == HIGH, off == LOW
  bool slow         = false;        // running at reduced speed
  int  speed        = 0;            // current motor speed, between -255 and 255
  int  targetSpeed  = 0;            // desired speed that should be reached  
  byte stepsBack    = 0;            // number of steps to be taken backwards
  
  long lastMillis   = 0;            // saves the current millis for the speedchange at each update 
  long reedMillis   = -999;         // saves the current millis for TOP position (2)
  
  bool nextStep();                  // returns true if a step has done
  void setPWM();                    // set the PWM output for the motor 
};

#endif
